@@ -19,6 +19,13 @@
     #include <windows.h>
     #include <wchar.h>
 #endif
+#ifdef __CYGWIN__
+    #include <sys/cygwin.h>  /* cygwin_conv_path */
+    #include <windows.h>  /* SetDllDirectoryW */
+    /* NOTE: SetDllDirectoryW is part of KERNEL32, which is automatically
+     * linked by Cygwin, so we do not need to explicitly link any
+     * win32 libraries. */
+#endif
 #include <stdio.h>  /* FILE */
 #include <stdlib.h> /* calloc */
 #include <string.h> /* memset */
@@ -36,22 +43,21 @@
 #include "pyi_pythonlib.h"
 #include "pyi_launch.h"
 #include "pyi_win32_utils.h"
+#include "pyi_splash.h"
 
 int
 pyi_main(int argc, char * argv[])
 {
     /*  archive_status contain status information of the main process. */
     ARCHIVE_STATUS *archive_status = NULL;
+    SPLASH_STATUS *splash_status = NULL;
     char executable[PATH_MAX];
     char homepath[PATH_MAX];
     char archivefile[PATH_MAX];
     int rc = 0;
+    int in_child = 0;
     char *extractionpath = NULL;
 
-#if defined(__linux__)
-    char *processname = NULL;
-#endif  /* defined(__linux__) */
-
 #ifdef _MSC_VER
     /* Visual C runtime incorrectly buffers stderr */
     setbuf(stderr, (char *)NULL);
@@ -81,6 +87,11 @@ pyi_main(int argc, char * argv[])
 
     extractionpath = pyi_getenv("_MEIPASS2");
 
+    /* NOTE: record the in-child status here, because extractionpath
+     * might get overwritten later on (on Windows and macOS, single
+     * process is used for --onedir mode). */
+    in_child = (extractionpath != NULL);
+
     /* If the Python program we are about to run invokes another PyInstaller
      * one-file program as subprocess, this subprocess must not be fooled into
      * thinking that it is already unpacked. Therefore, PyInstaller deletes
@@ -91,14 +102,18 @@ pyi_main(int argc, char * argv[])
 
     VS("LOADER: _MEIPASS2 is %s\n", (extractionpath ? extractionpath : "NULL"));
 
-    if ((! pyi_arch_setup(archive_status, executable)) &&
-        (! pyi_arch_setup(archive_status, archivefile))) {
+    /* Try opening the archive; first attempt to read it from executable
+     * itself (embedded mode), then from a stand-alone pkg file (sideload mode)
+     */
+    if ((! pyi_arch_setup(archive_status, executable, executable)) &&
+        (! pyi_arch_setup(archive_status, archivefile, executable))) {
             FATALERROR("Cannot open self %s or archive %s\n",
                        executable, archivefile);
             return -1;
     }
 
 #if defined(__linux__)
+    char *processname = NULL;
 
     /* Set process name on linux. The environment variable is set by
        parent launcher process. */
@@ -129,17 +144,60 @@ pyi_main(int argc, char * argv[])
 
 #endif
 
-#ifdef _WIN32
 
+#if defined(_WIN32) || defined(__CYGWIN__)
     if (extractionpath) {
         /* Add extraction folder to DLL search path */
-        wchar_t * dllpath_w;
-        dllpath_w = pyi_win32_utils_from_utf8(NULL, extractionpath, 0);
-        SetDllDirectory(dllpath_w);
-        VS("LOADER: SetDllDirectory(%s)\n", extractionpath);
-        free(dllpath_w);
+        wchar_t dllpath_w[PATH_MAX];
+#if defined(__CYGWIN__)
+        /* Cygwin */
+        if (cygwin_conv_path(CCP_POSIX_TO_WIN_W | CCP_RELATIVE, extractionpath, dllpath_w, PATH_MAX) != 0) {
+            FATAL_PERROR("cygwin_conv_path", "Failed to convert DLL search path!\n");
+            return -1;
+        }
+#else
+        /* Windows */
+        if (pyi_win32_utils_from_utf8(dllpath_w, extractionpath, PATH_MAX) == NULL) {
+            FATALERROR("Failed to convert DLL search path!\n");
+            return -1;
+        }
+#endif  /* defined(__CYGWIN__) */
+        VS("LOADER: SetDllDirectory(%S)\n", dllpath_w);
+        SetDllDirectoryW(dllpath_w);
+    }
+#endif  /* defined(_WIN32) || defined(__CYGWIN__) */
+
+    /*
+     * Check for splash screen resources.
+     * For the splash screen function to work PyInstaller
+     * needs to bundle tcl/tk with the application. This library
+     * is the same as tkinter uses.
+     */
+    splash_status = pyi_splash_status_new();
+
+    if (!in_child && pyi_splash_setup(splash_status, archive_status, NULL) == 0) {
+        /*
+         * Splash resources found, start splash screen
+         * If in onefile mode extract the required binaries
+         */
+        if ((!pyi_splash_extract(archive_status, splash_status)) &&
+            (!pyi_splash_attach(splash_status))) {
+            /* Everything was initialized, so it is safe to start
+             * the splash screen */
+            pyi_splash_start(splash_status, executable);
+        }
+        else {
+            /* Error attaching tcl/tk libraries.
+             * It may have happened that the libraries got (partly)
+             * loaded, so close them by finalizing the splash status */
+            pyi_splash_finalize(splash_status);
+            pyi_splash_status_free(&splash_status);
+        }
+    }
+    else {
+        /* No splash screen resources found */
+        pyi_splash_status_free(&splash_status);
     }
-#endif /* ifdef _WIN32 */
 
     if (extractionpath) {
         VS("LOADER: Already in the child - running user's code.\n");
@@ -161,19 +219,50 @@ pyi_main(int argc, char * argv[])
             strcpy(archive_status->mainpath, archive_status->temppath);
         }
 
+        /* On macOS in windowed mode, process Apple events and convert
+         * them to sys.argv - but only if we are in onedir mode! */
+#if defined(__APPLE__) && defined(WINDOWED)
+        if (!in_child) {
+            /* Initialize argc_pyi and argv_pyi with argc and argv */
+            if (pyi_utils_initialize_args(archive_status->argc, archive_status->argv) < 0) {
+                return -1;
+            }
+            /* Process Apple events; this updates argc_pyi/argv_pyi
+             * accordingly */
+            /* NOTE: processing Apple events swallows up the initial
+             * OAPP event, which seems to cause segmentation faults
+             * in tkinter-based frozen bundles made with Homebrew
+             * python 3.9 and Tcl/Tk 8.6.11. Until the exact cause
+             * is determined and addressed, this functionality must
+             * remain disabled.
+             */
+            /*pyi_process_apple_events(true);*/  /* short_timeout */
+            /* Update pointer to arguments */
+            pyi_utils_get_args(&archive_status->argc, &archive_status->argv);
+            /* TODO: do we need to de-register Apple event handlers before
+             * entering python? */
+        }
+#endif
+
         /* Main code to initialize Python and run user's code. */
         pyi_launch_initialize(archive_status);
         rc = pyi_launch_execute(archive_status);
         pyi_launch_finalize(archive_status);
 
+        /* Clean up splash screen resources; required when in single-process
+         * execution mode, i.e. when using --onedir on Windows or macOS. */
+        pyi_splash_finalize(splash_status);
+        pyi_splash_status_free(&splash_status);
+
+#if defined(__APPLE__) && defined(WINDOWED)
+        /* Clean up arguments that were used with Apple event processing .*/
+        pyi_utils_free_args();
+#endif
     }
     else {
-#if defined(__linux__)
-        char tmp_processname[16]; /* 16 bytes as per prctl() man page */
-#endif  /* defined(__linux__) */
 
         /* status->temppath is created if necessary. */
-        if (pyi_launch_extract_binaries(archive_status)) {
+        if (pyi_launch_extract_binaries(archive_status, splash_status)) {
             VS("LOADER: temppath is %s\n", archive_status->temppath);
             VS("LOADER: Error extracting binaries\n");
             return -1;
@@ -189,6 +278,7 @@ pyi_main(int argc, char * argv[])
         VS("LOADER: set _MEIPASS2 to %s\n", pyi_getenv("_MEIPASS2"));
 
 #if defined(__linux__)
+        char tmp_processname[16]; /* 16 bytes as per prctl() man page */
 
         /* Pass the process name to child via environment variable. */
         if (!prctl(PR_GET_NAME, tmp_processname, 0, 0)) {
@@ -212,6 +302,12 @@ pyi_main(int argc, char * argv[])
 
         VS("LOADER: Doing cleanup\n");
 
+        /* Finalize splash screen before temp directory gets wiped, since the splash
+         * screen might hold handles to shared libraries inside the temp dir. Those
+         * wouldn't be removed, leaving the temp folder behind. */
+        pyi_splash_finalize(splash_status);
+        pyi_splash_status_free(&splash_status);
+
         if (archive_status->has_temp_directory == true) {
             pyi_remove_temp_path(archive_status->temppath);
         }
