@@ -263,12 +263,12 @@ pyi_path_executable(char *execfile, const char *appname)
     /* GetModuleFileNameW returns an absolute, fully qualified path
      */
     if (!GetModuleFileNameW(NULL, modulename_w, PATH_MAX)) {
-        FATAL_WINERROR("GetModuleFileNameW", "Failed to get executable path.");
+        FATAL_WINERROR("GetModuleFileNameW", "Failed to get executable path.\n");
         return false;
     }
 
     if (!pyi_win32_utils_to_utf8(execfile, modulename_w, PATH_MAX)) {
-        FATALERROR("Failed to convert executable path to UTF-8.");
+        FATALERROR("Failed to convert executable path to UTF-8.\n");
         return false;
     }
 
@@ -290,12 +290,12 @@ pyi_path_executable(char *execfile, const char *appname)
     }
 
 #else /* ifdef _WIN32 */
-    /* On Linux, FreeBSD, and Solaris, we try these /proc paths first
+    /* On Linux, Cygwin, FreeBSD, and Solaris, we try these /proc paths first
      */
     size_t name_len = -1;
 
-    #if defined(__linux__)
-    name_len = readlink("/proc/self/exe", execfile, PATH_MAX-1);  /* Linux */
+    #if defined(__linux__) || defined(__CYGWIN__)
+    name_len = readlink("/proc/self/exe", execfile, PATH_MAX-1);  /* Linux, Cygwin */
     #elif defined(__FreeBSD__)
     name_len = readlink("/proc/curproc/file", execfile, PATH_MAX-1);  /* FreeBSD */
     #elif defined(__sun)
@@ -321,7 +321,7 @@ pyi_path_executable(char *execfile, const char *appname)
             char buffer[PATH_MAX];
             if (! pyi_search_path(buffer, appname)) {
                 /* Searching $PATH failed, user is crazy. */
-                VS("LOADER: Searching $PATH failed for %s", appname);
+                VS("LOADER: Searching $PATH failed for %s\n", appname);
                 if (snprintf(buffer, PATH_MAX, "%s", appname) >= PATH_MAX) {
                     VS("LOADER: Full path to application exceeds PATH_MAX: %s\n", appname);
                     return false;
@@ -376,10 +376,10 @@ pyi_path_archivefile(char *archivefile, const char *thisfile)
 FILE*
 pyi_path_fopen(const char* filename, const char* mode)
 {
-    wchar_t wfilename[MAX_PATH];
+    wchar_t wfilename[PATH_MAX];
     wchar_t wmode[10];
 
-    pyi_win32_utils_from_utf8(wfilename, filename, MAX_PATH);
+    pyi_win32_utils_from_utf8(wfilename, filename, PATH_MAX);
     pyi_win32_utils_from_utf8(wmode, mode, 10);
     return _wfopen(wfilename, wmode);
 }
