@@ -22,9 +22,6 @@
 
 #ifdef _WIN32
 
-/* windows.h will use API for WinServer 2003 with SP1 and WinXP with SP2 */
-#define _WIN32_WINNT 0x0502
-
 #include <windows.h>
 #include <commctrl.h> /* InitCommonControls */
 #include <stdio.h>    /* _fileno */
